@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/nodes.c,v 1.34 2009/11/14 19:39:19 amb Exp $
+ $Header: /home/amb/routino/src/RCS/nodes.c,v 1.38 2010/05/29 10:21:54 amb Exp $
 
  Node data type functions.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -50,9 +50,6 @@ Nodes *LoadNodeList(const char *filename)
 
  data=MapFile(filename);
 
- if(!data)
-    return(NULL);
-
  /* Copy the Nodes structure from the loaded data */
 
  *nodes=*((Nodes*)data);
@@ -60,8 +57,8 @@ Nodes *LoadNodeList(const char *filename)
  /* Adjust the pointers in the Nodes structure. */
 
  nodes->data=data;
- nodes->offsets=(index_t*)(data+(off_t)nodes->offsets);
- nodes->nodes=(Node*)(data+(off_t)nodes->nodes);
+ nodes->offsets=(index_t*)(data+sizeof(Nodes));
+ nodes->nodes=(Node*)(data+(sizeof(Nodes)+(nodes->latbins*nodes->lonbins+1)*sizeof(index_t)));
 
  return(nodes);
 }
@@ -317,59 +314,66 @@ Segment *FindClosestSegment(Nodes* nodes,Segments *segments,Ways *ways,double la
             {
              double lat1=latlong_to_radians(bin_to_latlong(nodes->latzero+latb)+off_to_latlong(nodes->nodes[i].latoffset));
              double lon1=latlong_to_radians(bin_to_latlong(nodes->lonzero+lonb)+off_to_latlong(nodes->nodes[i].lonoffset));
-             Segment *segment;
-             double dist1,dist2,dist3,dist3a,dist3b,distp;
+             distance_t dist1;
 
              dist1=Distance(lat1,lon1,latitude,longitude);
 
-             /* Check each segment for closeness and if valid for the profile */
-
-             segment=FirstSegment(segments,nodes,i);
-
-             do
+             if(dist1<distance)
                {
-                if(IsNormalSegment(segment))
+                Segment *segment;
+
+                /* Check each segment for closeness and if valid for the profile */
+
+                segment=FirstSegment(segments,nodes,i);
+
+                do
                   {
-                   Way *way=NULL;
-
-                   if(profile)
-                      way=LookupWay(ways,segment->way);
-
-                   if(!profile || way->allow&profile->allow)
+                   if(IsNormalSegment(segment))
                      {
-                      double lat2,lon2;
+                      Way *way=NULL;
 
-                      GetLatLong(nodes,OtherNode(segment,i),&lat2,&lon2);
+                      if(profile)
+                         way=LookupWay(ways,segment->way);
 
-                      dist2=Distance(lat2,lon2,latitude,longitude);
-                      dist3=Distance(lat1,lon1,lat2,lon2);
-
-                      /* Use law of cosines (assume flat Earth) */
-
-                      dist3a=(dist1*dist1-dist2*dist2+dist3*dist3)/(2*dist3);
-                      dist3b=dist3-dist3a;
-
-                      if(dist3a>=0 && dist3b>=0)
+                      if(!profile || way->allow&profile->allow)
                         {
-                         distp=sqrt(dist1*dist1-dist3a*dist3a);
+                         distance_t dist2,dist3;
+                         double lat2,lon2,dist3a,dist3b,distp;
 
-                         if((distance_t)distp<bestd)
+                         GetLatLong(nodes,OtherNode(segment,i),&lat2,&lon2);
+
+                         dist2=Distance(lat2,lon2,latitude,longitude);
+
+                         dist3=Distance(lat1,lon1,lat2,lon2);
+
+                         /* Use law of cosines (assume flat Earth) */
+
+                         dist3a=((double)dist1*(double)dist1-(double)dist2*(double)dist2+(double)dist3*(double)dist3)/(2.0*(double)dist3);
+                         dist3b=(double)dist3-dist3a;
+
+                         if(dist3a>=0 && dist3b>=0)
                            {
-                            bests=segment;
-                            bestn1=i;
-                            bestn2=OtherNode(segment,i);
-                            bestd1=(distance_t)dist3a;
-                            bestd2=(distance_t)dist3b;
-                            bestd=(distance_t)distp;
+                            distp=sqrt((double)dist1*(double)dist1-dist3a*dist3a);
+
+                            if(distp<(double)bestd)
+                              {
+                               bests=segment;
+                               bestn1=i;
+                               bestn2=OtherNode(segment,i);
+                               bestd1=(distance_t)dist3a;
+                               bestd2=(distance_t)dist3b;
+                               bestd=(distance_t)distp;
+                              }
                            }
                         }
                      }
-                  }
 
-                segment=NextSegment(segments,segment,i);
+                   segment=NextSegment(segments,segment,i);
+                  }
+                while(segment);
                }
-             while(segment);
-            }
+
+            } /* dist1 < distance */
 
           count++;
          }
