@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.82 2009/11/25 15:00:37 amb Exp $
+ $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.84 2010/05/30 12:50:40 amb Exp $
 
  Routing optimiser.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -144,9 +144,9 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
        segment_pref=profile->highway[HIGHWAY(way->type)];
 
        for(i=1;i<Property_Count;i++)
-          if(ways->props && PROPERTIES(i))
+          if(ways->props & PROPERTIES(i))
             {
-             if(way->props && PROPERTIES(i))
+             if(way->props & PROPERTIES(i))
                 segment_pref*=profile->props_yes[i];
              else
                 segment_pref*=profile->props_no[i];
@@ -369,9 +369,9 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
        segment_pref=profile->highway[HIGHWAY(way->type)];
 
        for(i=1;i<Property_Count;i++)
-          if(ways->props && PROPERTIES(i))
+          if(ways->props & PROPERTIES(i))
             {
-             if(way->props && PROPERTIES(i))
+             if(way->props & PROPERTIES(i))
                 segment_pref*=profile->props_yes[i];
              else
                 segment_pref*=profile->props_no[i];
@@ -473,7 +473,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
  if(!option_quiet)
    {
-    printf("\rRouted: Super-Nodes checked = %d\n",results->number);
+    printf("\rRouted: Super-Nodes checked = %d \n",results->number);
     fflush(stdout);
    }
 
@@ -605,9 +605,9 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
        segment_pref=profile->highway[HIGHWAY(way->type)];
 
        for(i=1;i<Property_Count;i++)
-          if(ways->props && PROPERTIES(i))
+          if(ways->props & PROPERTIES(i))
             {
-             if(way->props && PROPERTIES(i))
+             if(way->props & PROPERTIES(i))
                 segment_pref*=profile->props_yes[i];
              else
                 segment_pref*=profile->props_no[i];
@@ -760,9 +760,9 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
        segment_pref=profile->highway[HIGHWAY(way->type)];
 
        for(i=1;i<Property_Count;i++)
-          if(ways->props && PROPERTIES(i))
+          if(ways->props & PROPERTIES(i))
             {
-             if(way->props && PROPERTIES(i))
+             if(way->props & PROPERTIES(i))
                 segment_pref*=profile->props_yes[i];
              else
                 segment_pref*=profile->props_no[i];
