@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/router.c,v 1.67 2010/01/15 19:48:46 amb Exp $
+ $Header: /home/amb/routino/src/RCS/router.c,v 1.82 2010/05/31 12:34:45 amb Exp $
 
  OSM router.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -29,6 +29,7 @@
 
 #include "types.h"
 #include "functions.h"
+#include "translations.h"
 #include "profiles.h"
 #include "nodes.h"
 #include "segments.h"
@@ -39,7 +40,7 @@
 #define NWAYPOINTS 99
 
 /*+ The maximum distance from the specified point to search for a node or segment (in km). +*/
-#define MAXSEARCH  5
+#define MAXSEARCH  1
 
 /*+ The minimum distance along a segment from a node to insert a fake node. (in km). +*/
 #define MINSEGMENT 0.005
@@ -54,10 +55,22 @@ static double point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
 /*+ The option not to print any progress information. +*/
 int option_quiet=0;
 
+/*+ The options to select the format of the output. +*/
+int option_html=0,option_gpx_track=0,option_gpx_route=0,option_text=0,option_text_all=0,option_none=0;
+
 /*+ The option to calculate the quickest route insted of the shortest. +*/
 int option_quickest=0;
 
 
+/* Local functions */
+
+static void print_usage(int detail);
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The main program for the router.
+  ++++++++++++++++++++++++++++++++++++++*/
+
 int main(int argc,char** argv)
 {
  Nodes    *OSMNodes;
@@ -65,76 +78,136 @@ int main(int argc,char** argv)
  Ways     *OSMWays;
  Results  *results[NWAYPOINTS+1]={NULL};
  int       point_used[NWAYPOINTS+1]={0};
- int       help_profile=0,help_profile_js=0,help_profile_pl=0;
- char     *dirname=NULL,*prefix=NULL,*filename;
+ int       help_profile=0,help_profile_xml=0,help_profile_json=0,help_profile_pl=0;
+ char     *dirname=NULL,*prefix=NULL;
+ char     *profiles=NULL,*profilename=NULL;
+ char     *translations=NULL,*language=NULL;
  int       exactnodes=0;
  Transport transport=Transport_None;
- Profile   profile;
+ Profile  *profile=NULL;
  index_t   start=NO_NODE,finish=NO_NODE;
  int       arg,point;
 
  /* Parse the command line arguments */
 
  if(argc<2)
-   {
-   usage:
+    print_usage(0);
 
-    fprintf(stderr,"Usage: router [--lon1=]<longitude> [--lat1=]<latitude>\n"
-                   "              [--lon2=]<longitude> [--lon2=]<latitude>\n"
-                   "              [ ... [--lon99=]<longitude> [--lon99=]<latitude>]\n"
-                   "              [--help | --help-profile | --help-profile-js | --help-profile-pl]\n"
-                   "              [--dir=<name>] [--prefix=<name>]\n"
-                   "              [--shortest | --quickest]\n"
-                   "              [--exact-nodes-only]\n"
-                   "              [--quiet]\n"
-                   "              [--transport=<transport>]\n"
-                   "              [--highway-<highway>=<preference> ...]\n"
-                   "              [--speed-<highway>=<speed> ...]\n"
-                   "              [--property-<property>=<preference> ...]\n"
-                   "              [--oneway=[0|1]]\n"
-                   "              [--weight=<weight>]\n"
-                   "              [--height=<height>] [--width=<width>] [--length=<length>]\n"
-                   "\n"
-                   "<transport> defaults to motorcar but can be set to:\n"
-                   "%s"
-                   "\n"
-                   "<highway> can be selected from:\n"
-                   "%s"
-                   "\n"
-                   "<property> can be selected from:\n"
-                   "%s"
-                   "\n"
-                   "<preference> is a preference expressed as a percentage\n"
-                   "<speed> is a speed in km/hour\n"
-                   "<weight> is a weight in tonnes\n"
-                   "<height>, <width>, <length> are dimensions in metres\n"
-                   "\n",
-            TransportList(),HighwayList(),PropertyList());
-
-    return(1);
-   }
-
- /* Get the transport type if specified and fill in the default profile */
+ /* Get the non-routing, general program options */
 
  for(arg=1;arg<argc;arg++)
-    if(!strncmp(argv[arg],"--transport=",12))
+   {
+    if(!strcmp(argv[arg],"--help"))
+       print_usage(1);
+    else if(!strcmp(argv[arg],"--help-profile"))
+       help_profile=1;
+    else if(!strcmp(argv[arg],"--help-profile-xml"))
+       help_profile_xml=1;
+    else if(!strcmp(argv[arg],"--help-profile-json"))
+       help_profile_json=1;
+    else if(!strcmp(argv[arg],"--help-profile-perl"))
+       help_profile_pl=1;
+    else if(!strncmp(argv[arg],"--dir=",6))
+       dirname=&argv[arg][6];
+    else if(!strncmp(argv[arg],"--prefix=",9))
+       prefix=&argv[arg][9];
+    else if(!strncmp(argv[arg],"--profiles=",11))
+       profiles=&argv[arg][11];
+    else if(!strncmp(argv[arg],"--translations=",15))
+       translations=&argv[arg][15];
+    else if(!strcmp(argv[arg],"--exact-nodes-only"))
+       exactnodes=1;
+    else if(!strcmp(argv[arg],"--quiet"))
+       option_quiet=1;
+    else if(!strcmp(argv[arg],"--output-html"))
+       option_html=1;
+    else if(!strcmp(argv[arg],"--output-gpx-track"))
+       option_gpx_track=1;
+    else if(!strcmp(argv[arg],"--output-gpx-route"))
+       option_gpx_route=1;
+    else if(!strcmp(argv[arg],"--output-text"))
+       option_text=1;
+    else if(!strcmp(argv[arg],"--output-text-all"))
+       option_text_all=1;
+    else if(!strcmp(argv[arg],"--output-none"))
+       option_none=1;
+    else if(!strncmp(argv[arg],"--profile=",10))
+       profilename=&argv[arg][10];
+    else if(!strncmp(argv[arg],"--language=",11))
+       language=&argv[arg][11];
+    else if(!strncmp(argv[arg],"--transport=",12))
       {
        transport=TransportType(&argv[arg][12]);
 
        if(transport==Transport_None)
-          goto usage;
+         print_usage(0);
       }
+    else
+       continue;
+
+    argv[arg]=NULL;
+   }
+
+ /* Load in the profiles */
 
  if(transport==Transport_None)
     transport=Transport_Motorcar;
 
- profile=*GetProfile(transport);
+ if(profiles)
+   {
+    if(!ExistsFile(profiles))
+      {
+       fprintf(stderr,"Error: The '--profiles' option specifies a file that does not exist.\n");
+       return(1);
+      }
+   }
+ else
+   {
+    if(ExistsFile(FileName(dirname,prefix,"profiles.xml")))
+       profiles=FileName(dirname,prefix,"profiles.xml");
+    else
+      {
+       fprintf(stderr,"Error: The '--profiles' option was not used and the default 'profiles.xml' does not exist.\n");
+       return(1);
+      }
+   }
+
+ if(profiles && ParseXMLProfiles(profiles))
+   {
+    fprintf(stderr,"Error: Cannot read the profiles in the file '%s'.\n",profiles);
+    return(1);
+   }
+
+ if(profilename)
+   {
+    profile=GetProfile(profilename);
+
+    if(!profile)
+      {
+       fprintf(stderr,"Error: Cannot find a profile called '%s' in '%s'.\n",profilename,profiles);
+       return(1);
+      }
+   }
+ else
+    profile=GetProfile(TransportName(transport));
+
+ if(!profile)
+   {
+    profile=(Profile*)calloc(1,sizeof(Profile));
+    profile->transport=transport;
+   }
 
  /* Parse the other command line arguments */
 
  for(arg=1;arg<argc;arg++)
    {
-    if(isdigit(argv[arg][0]) ||
+    if(!argv[arg])
+       continue;
+    else if(!strcmp(argv[arg],"--shortest"))
+       option_quickest=0;
+    else if(!strcmp(argv[arg],"--quickest"))
+       option_quickest=1;
+    else if(isdigit(argv[arg][0]) ||
        ((argv[arg][0]=='-' || argv[arg][0]=='+') && isdigit(argv[arg][1])))
       {
        for(point=1;point<=NWAYPOINTS;point++)
@@ -158,11 +231,11 @@ int main(int argc,char** argv)
         char *p=&argv[arg][6];
         while(isdigit(*p)) p++;
         if(*p++!='=')
-           goto usage;
+           print_usage(0);
  
         point=atoi(&argv[arg][5]);
         if(point>NWAYPOINTS || point_used[point]&1)
-           goto usage;
+           print_usage(0);
  
        point_lon[point]=degrees_to_radians(atof(p));
        point_used[point]+=1;
@@ -172,37 +245,17 @@ int main(int argc,char** argv)
         char *p=&argv[arg][6];
         while(isdigit(*p)) p++;
         if(*p++!='=')
-           goto usage;
+           print_usage(0);
  
         point=atoi(&argv[arg][5]);
         if(point>NWAYPOINTS || point_used[point]&2)
-           goto usage;
+           print_usage(0);
  
        point_lat[point]=degrees_to_radians(atof(p));
        point_used[point]+=2;
       }
-    else if(!strcmp(argv[arg],"--help"))
-       goto usage;
-    else if(!strcmp(argv[arg],"--help-profile"))
-       help_profile=1;
-    else if(!strcmp(argv[arg],"--help-profile-js"))
-       help_profile_js=1;
-    else if(!strcmp(argv[arg],"--help-profile-pl"))
-       help_profile_pl=1;
-    else if(!strncmp(argv[arg],"--dir=",6))
-       dirname=&argv[arg][6];
-    else if(!strncmp(argv[arg],"--prefix=",9))
-       prefix=&argv[arg][9];
-    else if(!strcmp(argv[arg],"--shortest"))
-       option_quickest=0;
-    else if(!strcmp(argv[arg],"--quickest"))
-       option_quickest=1;
-    else if(!strcmp(argv[arg],"--exact-nodes-only"))
-       exactnodes=1;
-    else if(!strcmp(argv[arg],"--quiet"))
-       option_quiet=1;
     else if(!strncmp(argv[arg],"--transport=",12))
-       ; /* Done this already*/
+       ; /* Done this already */
     else if(!strncmp(argv[arg],"--highway-",10))
       {
        Highway highway;
@@ -210,7 +263,7 @@ int main(int argc,char** argv)
        char *string;
 
        if(!equal)
-          goto usage;
+           print_usage(0);
 
        string=strcpy((char*)malloc(strlen(argv[arg])),argv[arg]+10);
        string[equal-argv[arg]-10]=0;
@@ -218,9 +271,9 @@ int main(int argc,char** argv)
        highway=HighwayType(string);
 
        if(highway==Way_Count)
-          goto usage;
+          print_usage(0);
 
-       profile.highway[highway]=atof(equal+1);
+       profile->highway[highway]=atof(equal+1);
 
        free(string);
       }
@@ -231,7 +284,7 @@ int main(int argc,char** argv)
        char *string;
 
        if(!equal)
-          goto usage;
+          print_usage(0);
 
        string=strcpy((char*)malloc(strlen(argv[arg])),argv[arg]+8);
        string[equal-argv[arg]-8]=0;
@@ -239,9 +292,9 @@ int main(int argc,char** argv)
        highway=HighwayType(string);
 
        if(highway==Way_Count)
-          goto usage;
+          print_usage(0);
 
-       profile.speed[highway]=kph_to_speed(atof(equal+1));
+       profile->speed[highway]=kph_to_speed(atof(equal+1));
 
        free(string);
       }
@@ -252,7 +305,7 @@ int main(int argc,char** argv)
        char *string;
 
        if(!equal)
-          goto usage;
+          print_usage(0);
 
        string=strcpy((char*)malloc(strlen(argv[arg])),argv[arg]+11);
        string[equal-argv[arg]-11]=0;
@@ -260,39 +313,45 @@ int main(int argc,char** argv)
        property=PropertyType(string);
 
        if(property==Way_Count)
-          goto usage;
+          print_usage(0);
 
-       profile.props_yes[property]=atof(equal+1);
+       profile->props_yes[property]=atof(equal+1);
 
        free(string);
       }
     else if(!strncmp(argv[arg],"--oneway=",9))
-       profile.oneway=!!atoi(&argv[arg][9]);
+       profile->oneway=!!atoi(&argv[arg][9]);
     else if(!strncmp(argv[arg],"--weight=",9))
-       profile.weight=tonnes_to_weight(atof(&argv[arg][9]));
+       profile->weight=tonnes_to_weight(atof(&argv[arg][9]));
     else if(!strncmp(argv[arg],"--height=",9))
-       profile.height=metres_to_height(atof(&argv[arg][9]));
+       profile->height=metres_to_height(atof(&argv[arg][9]));
     else if(!strncmp(argv[arg],"--width=",8))
-       profile.width=metres_to_width(atof(&argv[arg][8]));
+       profile->width=metres_to_width(atof(&argv[arg][8]));
     else if(!strncmp(argv[arg],"--length=",9))
-       profile.length=metres_to_length(atof(&argv[arg][9]));
+       profile->length=metres_to_length(atof(&argv[arg][9]));
     else
-       goto usage;
+       print_usage(0);
    }
 
  for(point=1;point<=NWAYPOINTS;point++)
     if(point_used[point]==1 || point_used[point]==2)
-       goto usage;
+       print_usage(0);
 
  if(help_profile)
    {
-    PrintProfile(&profile);
+    PrintProfile(profile);
 
     return(0);
    }
- else if(help_profile_js)
+ else if(help_profile_xml)
    {
-    PrintProfilesJS();
+    PrintProfilesXML();
+
+    return(0);
+   }
+ else if(help_profile_json)
+   {
+    PrintProfilesJSON();
 
     return(0);
    }
@@ -303,37 +362,42 @@ int main(int argc,char** argv)
     return(0);
    }
 
- UpdateProfile(&profile);
+ /* Load in the translations */
 
- /* Load in the data */
+ if(option_html==0 && option_gpx_track==0 && option_gpx_route==0 && option_text==0 && option_text_all==0 && option_none==0)
+    option_html=option_gpx_track=option_gpx_route=option_text=option_text_all=1;
 
- OSMNodes=LoadNodeList(filename=FileName(dirname,prefix,"nodes.mem"));
-
- if(!OSMNodes)
+ if(option_html || option_gpx_route || option_gpx_track)
    {
-    fprintf(stderr,"Error: Cannot open nodes file '%s'.\n",filename);
-    return(1);
+    if(translations && ExistsFile(translations))
+       ;
+    else if(!translations && ExistsFile(FileName(dirname,prefix,"translations.xml")))
+       translations=FileName(dirname,prefix,"translations.xml");
+
+    if(!translations && language)
+      {
+       fprintf(stderr,"Error: Cannot use '--language' option without reading some translations.\n");
+       return(1);
+      }
+
+    if(translations && ParseXMLTranslations(translations,language))
+      {
+       fprintf(stderr,"Error: Cannot read the translations in the file '%s'.\n",translations);
+       return(1);
+      }
    }
 
- OSMSegments=LoadSegmentList(filename=FileName(dirname,prefix,"segments.mem"));
+ /* Load in the data - Note: No error checking because Load*List() will call exit() in case of an error. */
 
- if(!OSMSegments)
+ OSMNodes=LoadNodeList(FileName(dirname,prefix,"nodes.mem"));
+
+ OSMSegments=LoadSegmentList(FileName(dirname,prefix,"segments.mem"));
+
+ OSMWays=LoadWayList(FileName(dirname,prefix,"ways.mem"));
+
+ if(UpdateProfile(profile,OSMWays))
    {
-    fprintf(stderr,"Error: Cannot open segments file '%s'.\n",filename);
-    return(1);
-   }
-
- OSMWays=LoadWayList(filename=FileName(dirname,prefix,"ways.mem"));
-
- if(!OSMWays)
-   {
-    fprintf(stderr,"Error: Cannot open ways file '%s'.\n",filename);
-    return(1);
-   }
-
- if(!(profile.allow & OSMWays->allow))
-   {
-    fprintf(stderr,"Error: Database was not generated for selected transport.\n");
+    fprintf(stderr,"Error: Profile is invalid or not compatible with database.\n");
     return(1);
    }
 
@@ -356,15 +420,16 @@ int main(int argc,char** argv)
 
     if(exactnodes)
       {
-       finish=FindClosestNode(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,&profile,&distmin);
+       finish=FindClosestNode(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,profile,&distmin);
       }
     else
       {
        distance_t dist1,dist2;
 
-       segment=FindClosestSegment(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,&profile,&distmin,&node1,&node2,&dist1,&dist2);
-
-       finish=CreateFakes(OSMNodes,point,segment,node1,node2,dist1,dist2);
+       if((segment=FindClosestSegment(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,profile,&distmin,&node1,&node2,&dist1,&dist2)))
+          finish=CreateFakes(OSMNodes,point,segment,node1,node2,dist1,dist2);
+       else
+          finish=NO_NODE;
       }
 
     if(finish==NO_NODE)
@@ -409,7 +474,7 @@ int main(int argc,char** argv)
       }
     else
       {
-       begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,start,&profile);
+       begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,start,profile);
 
        if(!begin)
          {
@@ -450,7 +515,7 @@ int main(int argc,char** argv)
          }
        else
          {
-          end=FindFinishRoutes(OSMNodes,OSMSegments,OSMWays,finish,&profile);
+          end=FindFinishRoutes(OSMNodes,OSMSegments,OSMWays,finish,profile);
 
           if(!end)
             {
@@ -461,7 +526,7 @@ int main(int argc,char** argv)
 
        /* Calculate the middle of the route */
 
-       superresults=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,begin,end,&profile);
+       superresults=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,begin,end,profile);
 
        FreeResultsList(begin);
        FreeResultsList(end);
@@ -472,7 +537,7 @@ int main(int argc,char** argv)
           return(1);
          }
 
-       results[point]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,&profile);
+       results[point]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,profile);
 
        FreeResultsList(superresults);
       }
@@ -480,11 +545,8 @@ int main(int argc,char** argv)
 
  /* Print out the combined route */
 
- PrintRouteHead(FileName(dirname,prefix,"copyright.txt"));
-
- PrintRoute(results,NWAYPOINTS,OSMNodes,OSMSegments,OSMWays,&profile);
-
- PrintRouteTail();
+ if(!option_none)
+    PrintRoute(results,NWAYPOINTS,OSMNodes,OSMSegments,OSMWays,profile);
 
  return(0);
 }
@@ -652,3 +714,98 @@ Segment *ExtraFakeSegment(index_t node,index_t fakenode)
 
  return(NULL);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Print out the usage information.
+
+  int detail The level of detail to use - 0 = low, 1 = high.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void print_usage(int detail)
+{
+ fprintf(stderr,
+         "Usage: router [--help | --help-profile | --help-profile-xml |\n"
+         "                        --help-profile-json | --help-profile-perl ]\n"
+         "              [--dir=<dirname>] [--prefix=<name>]\n"
+         "              [--profiles=<filename>] [--translations=<filename>]\n"
+         "              [--exact-nodes-only]\n"
+         "              [--quiet]\n"
+         "              [--language=<lang>]\n"
+         "              [--output-html]\n"
+         "              [--output-gpx-track] [--output-gpx-route]\n"
+         "              [--output-text] [--output-text-all]\n"
+         "              [--output-none]\n"
+         "              [--profile=<name>]\n"
+         "              [--transport=<transport>]\n"
+         "              [--shortest | --quickest]\n"
+         "              --lon1=<longitude> --lat1=<latitude>\n"
+         "              --lon2=<longitude> --lon2=<latitude>\n"
+         "              [ ... --lon99=<longitude> --lon99=<latitude>]\n"
+         "              [--highway-<highway>=<preference> ...]\n"
+         "              [--speed-<highway>=<speed> ...]\n"
+         "              [--property-<property>=<preference> ...]\n"
+         "              [--oneway=(0|1)]\n"
+         "              [--weight=<weight>]\n"
+         "              [--height=<height>] [--width=<width>] [--length=<length>]\n");
+
+ if(detail)
+    fprintf(stderr,
+            "\n"
+            "--help                  Prints this information.\n"
+            "--help-profile          Prints the information about the selected profile.\n"
+            "--help-profile-xml      Prints all loaded profiles in XML format.\n"
+            "--help-profile-json     Prints all loaded profiles in JSON format.\n"
+            "--help-profile-perl     Prints all loaded profiles in Perl format.\n"
+            "\n"
+            "--dir=<dirname>         The directory containing the routing database.\n"
+            "--prefix=<name>         The filename prefix for the routing database.\n"
+            "--profiles=<filename>   The name of the profiles (defaults to 'profiles.xml'\n"
+            "                        with '--dirname' and '--prefix' options).\n"
+            "--translations=<fname>  The filename of the translations (defaults to\n"
+            "                         'translations.xml' with '--dirname' and '--prefix').\n"
+            "\n"
+            "--exact-nodes-only      Only route between nodes (don't find closest segment).\n"
+            "\n"
+            "--quiet                 Don't print any screen output when running.\n"
+            "--language=<lang>       Use the translations for specified language.\n"
+            "--output-html           Write an HTML description of the route.\n"
+            "--output-gpx-track      Write a GPX track file with all route points.\n"
+            "--output-gpx-route      Write a GPX route file with interesting junctions.\n"
+            "--output-text           Write a plain text file with interesting junctions.\n"
+            "--output-text-all       Write a plain test file with all route points.\n"
+            "--output-none           Don't write any output files or read any translations.\n"
+            "                        (If no output option is given then all are written.)\n"
+            "\n"
+            "--profile=<name>        Select the loaded profile with this name.\n"
+            "--transport=<transport> Select the transport to use (selects the profile\n"
+            "                        named after the transport if '--profile' is not used.)\n"
+            "\n"
+            "--shortest              Find the shortest route between the waypoints.\n"
+            "--quickest              Find the quickest route between the waypoints.\n"
+            "\n"
+            "--lon<n>=<longitude>    Specify the longitude of the n'th waypoint.\n"
+            "--lat<n>=<latitude>     Specify the latitude of the n'th waypoint.\n"
+            "\n"
+            "                                   Routing preference options\n"
+            "--highway-<highway>=<preference>   * preference for highway type (%%).\n"
+            "--speed-<highway>=<speed>          * speed for highway type (km/h).\n"
+            "--property-<property>=<preference> * preference for proprty type (%%).\n"
+            "--oneway=(0|1)                     * oneway streets are to be obeyed.\n"
+            "--weight=<weight>                  * maximum weight limit (tonnes).\n"
+            "--height=<height>                  * maximum height limit (metres).\n"
+            "--width=<width>                    * maximum width limit (metres).\n"
+            "--length=<length>                  * maximum length limit (metres).\n"
+            "\n"
+            "<transport> defaults to motorcar but can be set to:\n"
+            "%s"
+            "\n"
+            "<highway> can be selected from:\n"
+            "%s"
+            "\n"
+            "<property> can be selected from:\n"
+            "%s",
+            TransportList(),HighwayList(),PropertyList());
+
+ exit(!detail);
+}
